package river
Page
Library
Module
Module type
Parameter
Class
Class type
Module RiverSource
The source of a feed.
fetch ?timeout ?user_agent source returns an Atom or RSS feed from a source.
posts feeds is the list of deduplicated posts of the given feeds.
date post is the date of the post.
meta_description ?timeout ?user_agent post is the meta description of the post on the origin site.
To get the meta description, we make get the content of link post and look for an HTML meta tag with the name "description" or "og:description".
seo_image ?timeout ?user_agent post is the image to be used by social networks and links to the post.
To get the seo image, we make get the content of link post and look for an HTML meta tag with the name "og:image" or "twitter:image".
create_atom_feed posts creates a list of atom entries, which can then be used to create an atom feed that is an aggregate of the posts.